@@ -1,4 +1,5 @@
 use crate::{
+    copyless::VecHelper,
     util::{get, le_u32},
     Ck3Flavor,
 };
@@ -159,7 +160,7 @@ where
         let val = get::<4>(data)
             .map(u32::from_le_bytes)
             .ok_or_else(Error::eof)?;
-        self.token_tape.push(BinaryToken::U32(val));
+        self.token_tape.alloc().init(BinaryToken::U32(val));
         Ok(&data[4..])
     }
 
@@ -168,7 +169,7 @@ where
         let val = get::<8>(data)
             .map(u64::from_le_bytes)
             .ok_or_else(Error::eof)?;
-        self.token_tape.push(BinaryToken::U64(val));
+        self.token_tape.alloc().init(BinaryToken::U64(val));
         Ok(&data[8..])
     }
 
@@ -177,7 +178,7 @@ where
         let val = get::<4>(data)
             .map(i32::from_le_bytes)
             .ok_or_else(Error::eof)?;
-        self.token_tape.push(BinaryToken::I32(val));
+        self.token_tape.alloc().init(BinaryToken::I32(val));
         Ok(&data[4..])
     }
 
@@ -186,7 +187,7 @@ where
         let val = get::<4>(data)
             .map(|x| self.flavor.visit_f32(x))
             .ok_or_else(Error::eof)?;
-        self.token_tape.push(BinaryToken::F32(val));
+        self.token_tape.alloc().init(BinaryToken::F32(val));
         Ok(&data[4..])
     }
 
@@ -195,14 +196,14 @@ where
         let val = get::<8>(data)
             .map(|x| self.flavor.visit_f64(x))
             .ok_or_else(Error::eof)?;
-        self.token_tape.push(BinaryToken::F64(val));
+        self.token_tape.alloc().init(BinaryToken::F64(val));
         Ok(&data[8..])
     }
 
     #[inline]
     fn parse_bool(&mut self, data: &'a [u8]) -> Result<&'a [u8], Error> {
         let val = data.get(0).map(|&x| x != 0).ok_or_else(Error::eof)?;
-        self.token_tape.push(BinaryToken::Bool(val));
+        self.token_tape.alloc().init(BinaryToken::Bool(val));
         Ok(&data[1..])
     }
 
@@ -215,7 +216,7 @@ where
                 b: le_u32(&x[16..]),
             })
             .ok_or_else(Error::eof)?;
-        self.token_tape.push(BinaryToken::Rgb(val));
+        self.token_tape.alloc().init(BinaryToken::Rgb(val));
         Ok(&data[22..])
     }
 
@@ -236,17 +237,137 @@ where
     #[inline]
     fn parse_quoted_string(&mut self, data: &'a [u8]) -> Result<&'a [u8], Error> {
         let (scalar, rest) = self.parse_string_inner(data)?;
-        self.token_tape.push(BinaryToken::Quoted(scalar));
+        self.token_tape.alloc().init(BinaryToken::Quoted(scalar));
         Ok(rest)
     }
 
     #[inline]
     fn parse_unquoted_string(&mut self, data: &'a [u8]) -> Result<&'a [u8], Error> {
         let (scalar, rest) = self.parse_string_inner(data)?;
-        self.token_tape.push(BinaryToken::Unquoted(scalar));
+        self.token_tape.alloc().init(BinaryToken::Unquoted(scalar));
         Ok(rest)
     }
 
+    #[inline(never)]
+    fn fix_hidden_error(
+        &mut self,
+        data: &'a [u8],
+        array_ind_of_hidden_obj: &mut Option<usize>,
+        parent_ind: &mut usize,
+    ) -> Result<(), Error> {
+        // the `is_some` + `unwrap` approach is consistently 8-10% faster
+        // on the eu4 benchmark and I'm not sure why
+        let array_ind = array_ind_of_hidden_obj.take().unwrap();
+
+        // before we error, we should check if we previously parsed an empty array
+        // `history={{} 1444.11.11={core=AAA}}`
+        // so we're going to go back up the stack until we see our parent object
+        // and ensure that everything along the way is an empty array
+
+        let mut start = self.token_tape.len() - 3;
+        while start > array_ind {
+            match self.token_tape[start] {
+                BinaryToken::End(x) if x == start - 1 => {
+                    start -= 2;
+                }
+                _ => {
+                    return Err(Error::new(ErrorKind::InvalidSyntax {
+                        offset: self.offset(data) - 2,
+                        msg: String::from("nested values inside a hidden object are unsupported"),
+                    }));
+                }
+            }
+        }
+
+        let empty_objects_to_remove = self.token_tape.len() - 2 - array_ind;
+
+        let grand_ind = match self.token_tape[array_ind] {
+            BinaryToken::Array(x) => x,
+            _ => 0,
+        };
+
+        for _ in 0..empty_objects_to_remove {
+            self.token_tape.remove(self.token_tape.len() - 3);
+        }
+
+        *parent_ind = array_ind;
+        self.token_tape[*parent_ind] = BinaryToken::Object(grand_ind);
+
+        Ok(())
+    }
+
+    #[inline(never)]
+    fn pop_hidden_object(
+        &mut self,
+        state: &mut ParseState,
+        end_idx: usize,
+        array_ind_of_hidden_obj: &mut Option<usize>,
+        parent_ind: &mut usize,
+    ) {
+        let array_ind = array_ind_of_hidden_obj.take().unwrap();
+
+        self.token_tape[*parent_ind] = BinaryToken::HiddenObject(end_idx);
+        let end_idx = self.token_tape.len();
+        self.token_tape.alloc().init(BinaryToken::End(array_ind));
+
+        // Grab the grand parent from the outer array. Even though the logic should
+        // be more strict (ie: throwing an error when if the parent array index doesn't exist,
+        // or if the parent doesn't exist), but since hidden objects are such a rather rare
+        // occurrence, it's better to be flexible
+        let grand_ind = if let Some(parent) = self.token_tape.get_mut(array_ind) {
+            let grand_ind = match parent {
+                BinaryToken::Array(x) => *x,
+                _ => 0,
+            };
+            *parent = BinaryToken::Array(end_idx);
+            grand_ind
+        } else {
+            0
+        };
+
+        *state = match self.token_tape.get(grand_ind) {
+            Some(BinaryToken::Array(_x)) => ParseState::ArrayValue,
+            Some(BinaryToken::Object(_x)) => ParseState::Key,
+            _ => ParseState::Key,
+        };
+
+        *parent_ind = grand_ind;
+    }
+
+    #[inline(never)]
+    fn push_hidden_object(
+        &mut self,
+        data: &'a [u8],
+        array_ind_of_hidden_obj: &mut Option<usize>,
+        parent_ind: &mut usize,
+    ) -> Result<(), Error> {
+        // CK3 introduced hidden object inside lists so we work around it by trying to
+        // make the object explicit, but we first check to see if we have any prior
+        // array values
+        if self.token_tape.len() - *parent_ind <= 1
+            || matches!(
+                self.token_tape[self.token_tape.len() - 1],
+                BinaryToken::End(_)
+            )
+        {
+            return Err(Error::new(ErrorKind::InvalidSyntax {
+                msg: String::from("hidden object must start with a key"),
+                offset: self.offset(data),
+            }));
+        }
+
+        if *parent_ind + 2 == self.token_tape.len() {
+            return Ok(());
+        }
+
+        let hidden_object = BinaryToken::Object(*parent_ind);
+        *array_ind_of_hidden_obj = Some(*parent_ind);
+        *parent_ind = self.token_tape.len() - 1;
+        self.token_tape
+            .insert(self.token_tape.len() - 1, hidden_object);
+        Ok(())
+    }
+
     fn parse(&mut self) -> Result<(), Error> {
         let mut data = self.data;
         let mut state = ParseState::Key;
@@ -318,49 +439,15 @@ where
                 OPEN => {
                     if state == ParseState::ObjectValue {
                         if array_ind_of_hidden_obj.is_some() {
-                            // the `is_some` + `unwrap` approach is consistently 8-10% faster
-                            // on the eu4 benchmark and I'm not sure why
-                            let array_ind = array_ind_of_hidden_obj.take().unwrap();
-
-                            // before we error, we should check if we previously parsed an empty array
-                            // `history={{} 1444.11.11={core=AAA}}`
-                            // so we're going to go back up the stack until we see our parent object
-                            // and ensure that everything along the way is an empty array
-
-                            let mut start = self.token_tape.len() - 3;
-                            while start > array_ind {
-                                match self.token_tape[start] {
-                                    BinaryToken::End(x) if x == start - 1 => {
-                                        start -= 2;
-                                    }
-                                    _ => {
-                                        return Err(Error::new(ErrorKind::InvalidSyntax {
-                                            offset: self.offset(data) - 2,
-                                            msg: String::from(
-                                                "nested values inside a hidden object are unsupported",
-                                            ),
-                                        }));
-                                    }
-                                }
-                            }
-
-                            let empty_objects_to_remove = self.token_tape.len() - 2 - array_ind;
-
-                            let grand_ind = match self.token_tape[array_ind] {
-                                BinaryToken::Array(x) => x,
-                                _ => 0,
-                            };
-
-                            for _ in 0..empty_objects_to_remove {
-                                self.token_tape.remove(self.token_tape.len() - 3);
-                            }
-
-                            parent_ind = array_ind;
-                            self.token_tape[parent_ind] = BinaryToken::Object(grand_ind);
+                            self.fix_hidden_error(
+                                data,
+                                &mut array_ind_of_hidden_obj,
+                                &mut parent_ind,
+                            )?;
                         }
 
                         let ind = self.token_tape.len();
-                        self.token_tape.push(BinaryToken::Array(0));
+                        self.token_tape.alloc().init(BinaryToken::Array(0));
 
                         data = d;
                         let (d, token_id) = self.parse_next_id(data)?;
@@ -377,7 +464,7 @@ where
                                 };
 
                                 self.token_tape[ind] = BinaryToken::Array(ind + 1);
-                                self.token_tape.push(BinaryToken::End(ind));
+                                self.token_tape.alloc().init(BinaryToken::End(ind));
                                 continue;
                             }
 
@@ -421,7 +508,7 @@ where
                                 data = self.parse_rgb(data)?;
                             }
                             x => {
-                                self.token_tape.push(BinaryToken::Token(x));
+                                self.token_tape.alloc().init(BinaryToken::Token(x));
                             }
                         }
 
@@ -448,7 +535,7 @@ where
                         }
                     } else if state == ParseState::ArrayValue {
                         let ind = self.token_tape.len();
-                        self.token_tape.push(BinaryToken::Array(0));
+                        self.token_tape.alloc().init(BinaryToken::Array(0));
                         let (d, token_id) = self.parse_next_id(d)?;
                         data = d;
 
@@ -462,7 +549,7 @@ where
                                 };
 
                                 self.token_tape[ind] = BinaryToken::Array(ind + 1);
-                                self.token_tape.push(BinaryToken::End(ind));
+                                self.token_tape.alloc().init(BinaryToken::End(ind));
                                 continue;
                             }
 
@@ -470,7 +557,7 @@ where
                             OPEN => {
                                 self.token_tape[ind] = BinaryToken::Array(parent_ind);
                                 parent_ind = self.token_tape.len();
-                                self.token_tape.push(BinaryToken::Array(ind));
+                                self.token_tape.alloc().init(BinaryToken::Array(ind));
                                 state = ParseState::ArrayValue;
                                 continue;
                             }
@@ -503,7 +590,7 @@ where
                                 data = self.parse_rgb(data)?;
                             }
                             x => {
-                                self.token_tape.push(BinaryToken::Token(x));
+                                self.token_tape.alloc().init(BinaryToken::Token(x));
                             }
                         }
 
@@ -568,35 +655,14 @@ where
                             }));
                         }
 
-                        self.token_tape.push(BinaryToken::End(parent_ind));
-                        if let Some(array_ind) = array_ind_of_hidden_obj.take() {
-                            self.token_tape[parent_ind] = BinaryToken::HiddenObject(end_idx);
-                            let end_idx = self.token_tape.len();
-                            self.token_tape.push(BinaryToken::End(array_ind));
-
-                            // Grab the grand parent from the outer array. Even though the logic should
-                            // be more strict (ie: throwing an error when if the parent array index doesn't exist,
-                            // or if the parent doesn't exist), but since hidden objects are such a rather rare
-                            // occurrence, it's better to be flexible
-                            let grand_ind = if let Some(parent) = self.token_tape.get_mut(array_ind)
-                            {
-                                let grand_ind = match parent {
-                                    BinaryToken::Array(x) => *x,
-                                    _ => 0,
-                                };
-                                *parent = BinaryToken::Array(end_idx);
-                                grand_ind
-                            } else {
-                                0
-                            };
-
-                            state = match self.token_tape.get(grand_ind) {
-                                Some(BinaryToken::Array(_x)) => ParseState::ArrayValue,
-                                Some(BinaryToken::Object(_x)) => ParseState::Key,
-                                _ => ParseState::Key,
-                            };
-
-                            parent_ind = grand_ind;
+                        self.token_tape.alloc().init(BinaryToken::End(parent_ind));
+                        if array_ind_of_hidden_obj.is_some() {
+                            self.pop_hidden_object(
+                                &mut state,
+                                end_idx,
+                                &mut array_ind_of_hidden_obj,
+                                &mut parent_ind,
+                            );
                         } else {
                             self.token_tape[parent_ind] = BinaryToken::Object(end_idx);
                             parent_ind = grand_ind;
@@ -616,7 +682,7 @@ where
 
                         let end_idx = self.token_tape.len();
                         self.token_tape[parent_ind] = BinaryToken::Array(end_idx);
-                        self.token_tape.push(BinaryToken::End(parent_ind));
+                        self.token_tape.alloc().init(BinaryToken::End(parent_ind));
                         parent_ind = grand_ind;
                     } else if state == ParseState::ObjectValue {
                         return Err(Error::new(ErrorKind::InvalidSyntax {
@@ -636,34 +702,9 @@ where
                         data = d;
                         state = ParseState::ObjectValue;
                     } else if state == ParseState::ArrayValue {
-                        // CK3 introduced hidden object inside lists so we work around it by trying to
-                        // make the object explicit, but we first check to see if we have any prior
-                        // array values
-                        if self.token_tape.len() - parent_ind <= 1
-                            || matches!(
-                                self.token_tape[self.token_tape.len() - 1],
-                                BinaryToken::End(_)
-                            )
-                        {
-                            return Err(Error::new(ErrorKind::InvalidSyntax {
-                                msg: String::from("hidden object must start with a key"),
-                                offset: self.offset(data),
-                            }));
-                        }
-
-                        if parent_ind + 2 == self.token_tape.len() {
-                            data = d;
-                            state = ParseState::ObjectValue;
-                            continue;
-                        }
-
-                        let hidden_object = BinaryToken::Object(parent_ind);
-                        array_ind_of_hidden_obj = Some(parent_ind);
-                        parent_ind = self.token_tape.len() - 1;
-                        self.token_tape
-                            .insert(self.token_tape.len() - 1, hidden_object);
-                        state = ParseState::ObjectValue;
+                        self.push_hidden_object(d, &mut array_ind_of_hidden_obj, &mut parent_ind)?;
                         data = d;
+                        state = ParseState::ObjectValue;
                     } else {
                         return Err(Error::new(ErrorKind::InvalidSyntax {
                             msg: String::from("EQUAL not valid for a key"),
@@ -673,7 +714,7 @@ where
                 }
                 x => {
                     data = d;
-                    self.token_tape.push(BinaryToken::Token(x));
+                    self.token_tape.alloc().init(BinaryToken::Token(x));
                     state = SCALAR_STATE_NEXT[state as usize];
                 }
             }
