@@ -2,8 +2,10 @@
 mod de;
 mod reader;
 mod tape;
+mod writer;
 
 #[cfg(feature = "derive")]
 pub use self::de::TextDeserializer;
 pub use self::reader::{ArrayReader, ObjectReader, Reader, ScalarReader, ValueReader};
 pub use self::tape::{Operator, TextTape, TextToken};
+pub use self::writer::*;
