@@ -16,6 +16,12 @@ pub enum Operator {
 
     /// A `>=` token
     GreaterThanEqual,
+
+    /// A `!=` token
+    NotEqual,
+
+    /// A `==` token
+    Exact,
 }
 
 impl Display for Operator {
@@ -25,6 +31,8 @@ impl Display for Operator {
             Operator::GreaterThan => f.write_str(">"),
             Operator::LessThanEqual => f.write_str("<="),
             Operator::GreaterThanEqual => f.write_str(">="),
+            Operator::NotEqual => f.write_str("!="),
+            Operator::Exact => f.write_str("=="),
         }
     }
 }
@@ -153,9 +161,12 @@ impl TextTapeParser {
             data,
             original_length: data.len(),
             token_tape,
+            utf8_bom: false,
         };
 
         state.parse()?;
+        tape.utf8_bom = state.utf8_bom;
+
         Ok(())
     }
 }
@@ -164,23 +175,25 @@ struct ParserState<'a, 'b> {
     data: &'a [u8],
     original_length: usize,
     token_tape: &'b mut Vec<TextToken<'a>>,
+    utf8_bom: bool,
 }
 
 /// Houses the tape of tokens that is extracted from plaintext data
 #[derive(Debug, Default)]
 pub struct TextTape<'a> {
     token_tape: Vec<TextToken<'a>>,
+    utf8_bom: bool,
 }
 
 impl<'a> TextTape<'a> {
     /// Creates a windows 1252 object reader from the parsed tape
     pub fn windows1252_reader(&self) -> ObjectReader<Windows1252Encoding> {
-        ObjectReader::new(&self, Windows1252Encoding::new())
+        ObjectReader::new(self, Windows1252Encoding::new())
     }
 
     /// Creates a utf-8 object reader from the parsed tape
     pub fn utf8_reader(&self) -> ObjectReader<Utf8Encoding> {
-        ObjectReader::new(&self, Utf8Encoding::new())
+        ObjectReader::new(self, Utf8Encoding::new())
     }
 }
 
@@ -288,7 +301,7 @@ fn parse_quote_scalar(d: &[u8]) -> Result<(Scalar, &[u8]), Error> {
     }
 
     // from memchr: "SSE2 is avalbale on all x86_64 targets, so no CPU feature detection is necessary"
-    unsafe { inner(&d) }
+    unsafe { inner(d) }
 }
 
 #[inline]
@@ -415,7 +428,7 @@ fn split_at_scalar(d: &[u8]) -> (Scalar, &[u8]) {
     }
 
     // from memchr: "SSE2 is avalbale on all x86_64 targets, so no CPU feature detection is necessary"
-    unsafe { inner(&d) }
+    unsafe { inner(d) }
 }
 
 impl<'a> TextTape<'a> {
@@ -438,6 +451,11 @@ impl<'a> TextTape<'a> {
     pub fn tokens(&self) -> &[TextToken<'a>] {
         self.token_tape.as_slice()
     }
+
+    /// Return if there was a UTF8 BOM in the data
+    pub fn utf8_bom(&self) -> bool {
+        self.utf8_bom
+    }
 }
 
 impl<'a, 'b> ParserState<'a, 'b> {
@@ -455,7 +473,7 @@ impl<'a, 'b> ParserState<'a, 'b> {
             let mut ptr = start_ptr;
             while ptr < end_ptr {
                 match *ptr {
-                    b' ' | b'\t' | b'\n' | b'\r' => {}
+                    b' ' | b'\t' | b'\n' | b'\r' | b';' => {}
                     b'#' => {
                         ptr = ptr.offset(1);
                         while ptr < end_ptr && *ptr != b'\n' {
@@ -531,9 +549,25 @@ impl<'a, 'b> ParserState<'a, 'b> {
         //
         // These are especially tricky, but essentially this function's job is to skip the equal
         // token (the 99.9% typical case) if possible.
-        if d[0] == b'=' {
+        let range = d.as_ptr_range();
+        let mut cur = range.start;
+        if unsafe { *cur } != b'=' {
+            return self.parse_key_value_separator_unlikely(d);
+        }
+
+        cur = unsafe { cur.add(1) };
+        if cur == range.end {
+            self.parse_key_value_separator_unlikely(d)
+        } else if unsafe { *cur } != b'=' {
             &d[1..]
-        } else if d[0] == b'<' {
+        } else {
+            self.parse_key_value_separator_unlikely(d)
+        }
+    }
+
+    #[inline(never)]
+    fn parse_key_value_separator_unlikely(&mut self, d: &'a [u8]) -> &'a [u8] {
+        if d[0] == b'<' {
             if d.get(1).map_or(false, |c| *c == b'=') {
                 self.token_tape
                     .push(TextToken::Operator(Operator::LessThanEqual));
@@ -553,6 +587,13 @@ impl<'a, 'b> ParserState<'a, 'b> {
                     .push(TextToken::Operator(Operator::GreaterThan));
                 &d[1..]
             }
+        } else if d[0] == b'!' && d.get(1).map_or(false, |c| *c == b'=') {
+            self.token_tape
+                .push(TextToken::Operator(Operator::NotEqual));
+            &d[2..]
+        } else if d[0] == b'=' && d.get(1).map_or(false, |c| *c == b'=') {
+            self.token_tape.push(TextToken::Operator(Operator::Exact));
+            &d[2..]
         } else {
             d
         }
@@ -564,6 +605,11 @@ impl<'a, 'b> ParserState<'a, 'b> {
         let mut data = self.data;
         let mut state = ParseState::Key;
 
+        self.utf8_bom = data.get(..3).map_or(false, |x| x == [0xef, 0xbb, 0xbf]);
+        if self.utf8_bom {
+            data = &data[3..];
+        }
+
         // This variable keeps track of outer array when we're parsing a hidden object.
         // A hidden object textually looks like:
         //     levels={ 10 0=2 1=2 }
@@ -583,10 +629,28 @@ impl<'a, 'b> ParserState<'a, 'b> {
             let d = match self.skip_ws_t(data) {
                 Some(d) => d,
                 None => {
-                    if parent_ind == 0 && state == ParseState::Key {
+                    if state != ParseState::Key {
+                        return Err(Error::eof());
+                    }
+
+                    if parent_ind == 0 {
                         return Ok(());
                     } else {
-                        return Err(Error::eof());
+                        // Support for files that don't have enough closing brackets (ugh)
+                        let grand_ind = match self.token_tape.get(parent_ind) {
+                            Some(TextToken::Array(x)) => *x,
+                            Some(TextToken::Object(x)) => *x,
+                            _ => 0,
+                        };
+
+                        if grand_ind == 0 {
+                            let end_idx = self.token_tape.len();
+                            self.token_tape.push(TextToken::End(parent_ind));
+                            self.token_tape[parent_ind] = TextToken::Object(end_idx);
+                            return Ok(());
+                        } else {
+                            return Err(Error::eof());
+                        }
                     }
                 }
             };
@@ -862,12 +926,11 @@ impl<'a, 'b> ParserState<'a, 'b> {
                     }
                 }
                 ParseState::FirstValue => match data[0] {
-                    b'=' => {
+                    b'=' | b'>' | b'<' => {
                         let ind = self.token_tape.len() - 2;
                         self.token_tape[ind] = TextToken::Object(parent_ind);
-                        data = &data[1..];
                         parent_ind = ind;
-                        state = ParseState::ObjectValue;
+                        state = ParseState::KeyValueSeparator;
                     }
                     _ => {
                         let ind = self.token_tape.len() - 2;
@@ -1026,7 +1089,7 @@ impl<'a, 'b> ParserState<'a, 'b> {
             return Err(Error::eof());
         }
 
-        let (scalar, data) = split_at_scalar(&data);
+        let (scalar, data) = split_at_scalar(data);
         if !matches!(data.get(0), Some(&x) if x == b']') {
             return Err(Error::new(ErrorKind::InvalidSyntax {
                 offset: self.offset(data),
@@ -1620,6 +1683,15 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_bom() {
+        let data = b"\xef\xbb\xbf#hello";
+        let tape = parse(&data[..]).unwrap();
+
+        assert_eq!(tape.token_tape, vec![]);
+        assert!(tape.utf8_bom);
+    }
+
     #[test]
     fn test_period_in_identifiers() {
         let data = b"flavor_tur.8=yes";
@@ -2051,6 +2123,50 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_not_equal_operator() {
+        let data = b"count != 2";
+        assert_eq!(
+            parse(&data[..]).unwrap().token_tape,
+            vec![
+                TextToken::Unquoted(Scalar::new(b"count")),
+                TextToken::Operator(Operator::NotEqual),
+                TextToken::Unquoted(Scalar::new(b"2")),
+            ]
+        );
+    }
+
+    #[test]
+    fn test_not_exact_operator() {
+        let data = b"start_date == 1066.9.15";
+        assert_eq!(
+            parse(&data[..]).unwrap().token_tape,
+            vec![
+                TextToken::Unquoted(Scalar::new(b"start_date")),
+                TextToken::Operator(Operator::Exact),
+                TextToken::Unquoted(Scalar::new(b"1066.9.15")),
+            ]
+        );
+    }
+
+    #[test]
+    fn test_operator_in_object() {
+        let data = b"value={ date < 1941.7.7 is_preparing=true }";
+        assert_eq!(
+            parse(&data[..]).unwrap().token_tape,
+            vec![
+                TextToken::Unquoted(Scalar::new(b"value")),
+                TextToken::Object(7),
+                TextToken::Unquoted(Scalar::new(b"date")),
+                TextToken::Operator(Operator::LessThan),
+                TextToken::Unquoted(Scalar::new(b"1941.7.7")),
+                TextToken::Unquoted(Scalar::new(b"is_preparing")),
+                TextToken::Unquoted(Scalar::new(b"true")),
+                TextToken::End(1),
+            ]
+        );
+    }
+
     #[test]
     fn test_extraneous_closing_bracket() {
         let data = b"foo = { bar } } ";
@@ -2115,6 +2231,26 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_missing_bracket() {
+        let data = br#"BRA_GAR_01 = { ordered = { 1 = { "%da Divisao de Infantaria" } }"#;
+        assert_eq!(
+            parse(&data[..]).unwrap().token_tape,
+            vec![
+                TextToken::Unquoted(Scalar::new(b"BRA_GAR_01")),
+                TextToken::Object(9),
+                TextToken::Unquoted(Scalar::new(b"ordered")),
+                TextToken::Object(8),
+                TextToken::Unquoted(Scalar::new(b"1")),
+                TextToken::Array(7),
+                TextToken::Quoted(Scalar::new(b"%da Divisao de Infantaria")),
+                TextToken::End(5),
+                TextToken::End(3),
+                TextToken::End(1),
+            ]
+        );
+    }
+
     #[test]
     fn test_unquoted_non_ascii() {
         // More vic2 shenanigans
@@ -2263,6 +2399,20 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_skip_semicolon() {
+        let data = b"value=\"win\"; a=b";
+        assert_eq!(
+            parse(&data[..]).unwrap().token_tape,
+            vec![
+                TextToken::Unquoted(Scalar::new(b"value")),
+                TextToken::Quoted(Scalar::new(b"win")),
+                TextToken::Unquoted(Scalar::new(b"a")),
+                TextToken::Unquoted(Scalar::new(b"b")),
+            ]
+        );
+    }
+
     #[test]
     fn test_parameter_eof() {
         let data = b"[[";
